@@ -4,7 +4,7 @@
  * Eddie Kohler
  *
  * Copyright (c) 1999-2000 Massachusetts Institute of Technology
- * Copyright (c) 2001-2004 Eddie Kohler
+ * Copyright (c) 2001-2006 Eddie Kohler
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -51,7 +51,7 @@ StringAccum::make_out_of_memory()
 {
   assert(_cap >= 0);
   delete[] _s;
-  _s = reinterpret_cast<unsigned char *>(const_cast<char *>(String::out_of_memory_string().data()));
+  _s = reinterpret_cast<unsigned char *>(const_cast<char *>(String::out_of_memory_data()));
   _cap = -1;
   _len = 0;
 }
