@@ -1,6 +1,6 @@
 /* main.cc -- driver for mmafm program
  *
- * Copyright (c) 1997-2004 Eddie Kohler
+ * Copyright (c) 1997-2006 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -245,7 +245,7 @@ Interpolation settings:\n\
   -p, --precision=N             Allow N digits of fraction (default 3).\n\
   -k, --min-kern=N              Remove kerns smaller than N (default 2).\n\
 \n\
-Report bugs to <kohler@icir.org>.\n", program_name);
+Report bugs to <kohler@cs.ucla.edu>.\n", program_name);
 }
 
 
@@ -321,7 +321,7 @@ main(int argc, char *argv[])
       
      case VERSION_OPT:
       printf("mmafm (LCDF typetools) %s\n", VERSION);
-      printf("Copyright (C) 1997-2004 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2006 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
